from bigdoorkit.resources.base import BDResource
from bigdoorkit.resources.user import EndUser

class NamedAwardCollection(BDResource):
    endpoint = "named_award_collection"

class NamedAward(BDResource):
    endpoint = "named_award"
    parent_class = NamedAwardCollection
    parent_id_attr = "named_award_collection_id"

    def __init__(self, **kw):
        self.named_award_collection_id = kw.get('named_award_collection_id', None)
        self.relative_weight = kw.get('relative_weight', None)
        self.collection_resource_uri = kw.get('collection_resource_uri', None)
        super(NamedAward, self).__init__(**kw)

class Award(BDResource):
    endpoint = "award"
    parent_class = EndUser
    parent_id_attr = "end_user_login"

    def __init__(self, **kw):
        self.end_user_login = kw.get('end_user_login', None)
        self.named_award_id = kw.get('named_award_id', None)
        super(Award, self).__init__(**kw)



from bigdoorkit.client import Client
from bigdoorkit.exc import MissingParentDetails
from datetime import datetime
from types import MethodType

def clean_obj_keys(data):
    """JSON returns unicode keys which cannot be passed to
    __init__ methods directly. Here we convert the keys to strings.
    """
    return dict([(str(k), v) for k, v in data.items()])

class BDResource(object):
    endpoint = None
    parent_class = None
    parent_id_attr = None

    def __init__(self,
                 created_timestamp,
                 modified_timestamp,
                 **kw):
        self.id = kw.get('id', None)
        self.pub_title = kw.get('pub_title', None)
        self.pub_description = kw.get('pub_description', None)
        self.end_user_title = kw.get('end_user_title', None)
        self.end_user_description = kw.get('end_user_description', None)

        if isinstance(created_timestamp, (int, float)):
            created_timestamp = datetime.fromtimestamp(created_timestamp)
        self.created_timestamp = created_timestamp

        if isinstance(modified_timestamp, (int, float)):
            modified_timestamp = datetime.fromtimestamp(modified_timestamp)
        self.modified_timestamp = modified_timestamp

        # make the delete method an instance method
        self.delete = MethodType(self.__instance_delete, self, self.__class__)

        self._undefined_kw = kw

    @classmethod
    def all(cls, client):
        """Retrieves all of the available objects"""
        if cls.parent_class and not getattr(cls, cls.parent_id_attr, None):
            raise MissingParentDetails()
        endpoint = cls.endpoint
        if cls.parent_class:
            endpoint = "%s/%s/%s" % (cls.parent_class.endpoint,
                                     getattr(cls, cls.parent_id_attr),
                                     cls.endpoint)
        data = client.get(endpoint)
        return [cls(**clean_obj_keys(i)) for i in data[0]]

    @classmethod
    def get(cls, id, client):
        """Retrieves an object with the given `id`, using the `client` object
        if passed.
        """
        data = client.get("%s/%s" % (cls.endpoint, id))
        return cls(**clean_obj_keys(data[0]))

    def save(self, client):
        """Create or Update this object.

        If the object has an id, this method will try to PUT, otherwise
        the HTTP method will be POST.
        """
        if not self.id:
            client.post(self.endpoint, self.to_dict())
        else:
            data = client.put(self.endpoint, self.to_dict())
            # XXX: This is pretty dirty. We'll see how it works out.
            self = self.__class__(**data)

    def __class_delete(cls, id, client=None):
        """Deletes an object identified by `id`"""
        if client is None:
            client = get_client()
        client.delete(cls.endpoint, id)

    delete = classmethod(__class_delete)

    def __instance_delete(self, obj, client=None):
        # XXX: may need to change to self.__class__.delete(self.id, client)
        return self.__class_delete(self.id, client)



from award import Award, NamedAward, NamedAwardCollection
from currency import Currency
from good import Good, NamedGood, NamedGoodCollection
from level import Level, NamedLevel, NamedLevelCollection
from user import EndUser

__all_ = ["Award",
          "NamedAward",
          "NamedAwardCollection",
          "Currency",
          "Good",
          "NamedGood",
          "NamedGoodCollection",
          "Level",
          "NamedLevel",
          "NamedLevelCollection",
          "EndUser"]



from bigdoorkit.resources.base import BDResource

class EndUser(BDResource):
    endpoint = "end_user"

    def __init__(self, **kw):
        self.guid = guid
        self.end_user_login = kw.get('end_user_login', None)
        super(EndUser, self).__init__(**kw)



from bigdoorkit.resources.base import BDResource

class Currency(BDResource):
    endpoint = "currency"

    def __init__(self, **kw):
        self.currency_type_id = kw.get('currency_type_id', None)
        self.currency_type_description = kw.get('currency_type_description', None)
        self.currency_type_title = kw.get('currency_type_title', None)
        self.exchange_rate = kw.get('exchange_rate', None)
        self.relative_weight = kw.get('relative_weight', None)
        super(Currency, self).__init__(**kw)



import restkit
import hashlib
try:
    import json
except ImportError, e:
    import simplejson as json

from uuid import uuid4
from time import time as unix_time
from urllib import urlencode
from exc import PayloadError

__all__ = ["Client"]

class Client(object):
    """Requires `app_secret` and `app_key` as supplied by BigDoor.
    Optional `api_host` parameter allows for use with API compatible
    services and/or staging servers.
    """
    def __init__(self, app_secret, app_key, api_host=None):
        """Constructor for a `Client` object.

        Parameters:
            - app_secret string The API secret supplied by BigDoor

            - app_key string The API key supplied by BigDoor

            - api_host string An alternative host to enable use with testing
            servers.
        """
        self.app_secret = app_secret
        self.app_key = app_key
        if not api_host:
            api_host = "http://api.bigdoor.com"
        self.api_host = api_host
        self.base_url = "/api/publisher/%s" % self.app_key
        self.conn = restkit.Resource(self.api_host)

    def generate_token(self):
        """Helper method
        """
        return uuid4().hex

    def generate_signature(self, url, params=None, payload=None):
        """Generates the appropriate signature given a url and optional
        params."""
        sig = url
        if params:
            sig += self._flatten_params(params)
        if payload:
            sig += self._flatten_params(payload)
        sig += self.app_secret
        return hashlib.sha256(sig).hexdigest()

    def _flatten_params(self, params):
        """Flattens a parameter dictionary for signature generation"""
        return ''.join(["%s%s" % (k, params[k])
                        for k in sorted(params.keys())
                        if k not in ('sig', 'format')])

    def _sign_request(self, method, url, params=None, payload=None):
        """Algorithm to sign a request as per the BigDoor documentation. Adds
        defaults to `params` and `payload` if not present.

        Parameters:
            - method string The HTTP method for the request

            - url string The full URL, including the base /api/publisher/[app_key]

            - params dict The parameters to be sent via the GET query string

            - payload dict The data to be sent via the POST body
        """
        if params is None:
            params = {}
        is_postish = method in ['post', 'put']

        # The payload time is assumed to be the correct value
        if is_postish and 'time' in payload:
            params['time'] = payload['time']
        if not 'time' in params:
            params['time'] = str(unix_time())
        if is_postish and not 'time' in payload:
            payload['time'] = params['time']

        if is_postish and not 'token' in payload:
            payload['token'] = self.generate_token()

        if method == 'delete' and not 'delete_token' in params:
            params['delete_token'] = self.generate_token()

        params['sig'] = self.generate_signature(url, params, payload)
        return params, payload

    def _abs_from_rel(self, url):
        """Private helper method to concatenate the base url and endpoint.
        """
        return "%s/%s" % (self.base_url, url)

    def do_request(self, method, endpoint, params=None, payload=None):
        """Sends a request to the API, signing it before it is sent.
        Returns a restkit response object.

        Parameters:
            - method string The HTTP method to use for the request. Can be one
            of ['get', 'delete', 'post', 'put']. Note: case of the string does
            not matter.

            - endpoint string The relative URI that comes directly after
            your API key in the BigDoor documentation.

            - params dict The parameters to be sent via the GET query string.

            - payload dict The data to be sent via the POST body
        """
        # Copy the parameters and payload variables so we don't pollute passed
        # dictionaries with auto-generated API information.
        par = {}
        pay = {}

        if params is not None:
            par = params.copy()
        if payload is not None:
            pay = payload.copy()

        method = method.lower()
        if method in ['post', 'put'] and not isinstance(pay, dict):
            err_msg = "Payload must be <type 'dict'>, not %s" % type(pay)
            raise PayloadError(err_msg)
        url = self._abs_from_rel(endpoint)
        par, pay = self._sign_request(method, url, par, pay)
        func = getattr(self.conn, method)
        print url
        if method in ['post', 'put']:
            par['payload'] = pay

        return func(url, **par)

    def get(self, endpoint, params=None):
        """Sends a GET request to the API and returns a native data
        structure from the JSON response.

        Parameters:
            - endpoint string The relative URI that comes directly after
            your API key in the BigDoor documentation.

            - params dict The parameters to be sent via the GET query string.
        """
        r = self.do_request('get', endpoint, params)
        return json.loads(r.body_string())

    def delete(self, endpoint, params=None):
        """Sends a DELETE request to the API.

        Parameters:
            - endpoint string The relative URI that comes directly after
            your API key in the BigDoor documentation.

            - params dict The parameters to be sent via the GET query string.
        """
        r = self.do_request('delete', endpoint, params)

    def post(self, endpoint, params=None, payload=None):
        """Sends a POST request to the API and returns a native data
        structure from the JSON response.

        Parameters:
            - endpoint string The relative URI that comes directly after
            your API key in the BigDoor documentation.

            - params dict The parameters to be sent via the GET query string.

            - payload dict The data to be sent via the POST body
        """
        r = self.do_request('post', endpoint, params, payload)
        return json.loads(r.body_string())

    def put(self, endpoint, params=None, payload=None):
        """Sends a PUT request to the API and returns a native data
        structure from the JSON response.

        Parameters:
            - endpoint string The relative URI that comes directly after
            your API key in the BigDoor documentation.

            - params dict The parameters to be sent via the GET query string.

            - payload dict The data to be sent via the POST body
        """
        r = self.do_request('put', endpoint, params, payload)
        return json.loads(r.body_string())



from bigdoorkit.resources.base import BDResource
from bigdoorkit.resources.user import EndUser

class NamedGoodCollection(BDResource):
    endpoint = "named_good_collection"

class NamedGood(BDResource):
    endpoint = "named_good"
    parent_class = NamedGoodCollection
    parent_id_attr = "named_good_collection_id"

    def __init__(self, **kw):
        self.named_good_collection_id = kw.get('named_good_collection_id', None)
        self.relative_weight = kw.get('relative_weight', None)
        self.collection_resource_uri = kw.get('collection_resource_uri', None)
        super(NamedGood, self).__init__(**kw)

class Good(BDResource):
    endpoint = "good"
    parent_class = EndUser
    parent_id_attr = "reciever_end_user_login"

    def __init__(self, **kw):
        self.sender_end_user_login = kw.get('sender_end_user_login', None)
        self.reciever_end_user_login = kw.get('reciever_end_user_login', None)
        self.named_good_id = kw.get('named_good_id', None)
        self.end_user_message_subject = kw.get('end_user_message_subject', None)
        self.end_user_message_body = kw.get('end_user_message_body', None)
        super(Good, self).__init__(**kw)



class BigdoorError(Exception):
    """The base type of Bigdoor Exception"""

class MissingClient(BigdoorError):
    """Raised when there is no client available to service
    the request.
    """

class MissingParentDetails(BigdoorError):
    """Raised when there are certain attributes missing regarding a resource's
    parent.
    """

class PayloadError(BigdoorError):
    """Raised when there's a problem with the payload variable"""



from setuptools import setup, find_packages
import os

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()
NEWS = open(os.path.join(here, 'NEWS.txt')).read()


version = '0.1.4'

install_requires = [
    'restkit',
]


setup(name='bigdoorkit',
    version=version,
    description="Client library for the BigDoor API, based on RESTkit",
    long_description=README + '\n\n' + NEWS,
    classifiers=[
      # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    keywords='',
    author='Lee McFadden',
    author_email='spleeman@gmail.com',
    url='http://bitbucket.org/splee/bigdoorkit',
    license='MIT',
    packages=find_packages('src'),
    package_dir = {'': 'src'},include_package_data=True,
    zip_safe=False,
    install_requires=install_requires,
    entry_points={}
)



from bigdoorkit.resources.base import BDResource
from bigdoorkit.resources.user import EndUser

class NamedLevelCollection(BDResource):
    endpoint = "named_level_collection"

    def __init__(self, **kw):
        self.currency_id = kw.get('currency_id', None)
        super(NamedLevelCollection, self).__init__(**kw)

class NamedLevel(BDResource):
    endpoint = "named_level"
    parent_class = NamedLevelCollection
    parent_id_attr = "named_level_collection_id"

    def __init__(self, **kw):
        self.named_level_collection_id = kw.get('named_level_collection_id', None)
        self.threshold = kw.get('threshold', None)
        self.collection_resource_uri = kw.get('collection_resource_uri', None)
        super(NamedLevel, self).__init__(**kw)

class Level(BDResource):
    endpoint = "level"
    parent_class = EndUser
    parent_id_attr = "end_user_login"

    def __init__(self, **kw):
        self.end_user_login = kw.get('end_user_login', None)
        self.named_level_id = kw.get('named_level_id', None)
        self.transaction_group_id = kw.get('transaction_group_id', None)
        self.next_level_uri = kw.get('next_level_uri', None)
        self.previous_level_uri = kw.get('previous_level_uri', None)
        super(Level, self).__init__(**kw)
